# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Common functions for getting various kinds of data."""

from Common import *

import Database
import XMLParse

def get_league_season(league_id):
    """Returns the current season of a league.

     - league_id -- The league id of the league to get season from."""

    data = XMLParse.get(["world details"], ["LeagueList.League.xml_list",
                                            "LeagueList.League.xml_attrs.Index"])

    lindeces = data.get("LeagueList.League.xml_list")
    if lindeces is None:
        lindeces = data.get("LeagueList.League.xml_attrs.Index")
        if lindeces is None:
            return
        else:
            lindeces = [str(lindeces)]
    else:
        lindeces = ["+" + x for x in lindeces.split(",")]

    paths = []
    for index in lindeces:
        paths.append("LeagueList.League%s.LeagueID" % index)
        paths.append("LeagueList.League%s.Season" % index)

    data = XMLParse.get(["world details"], paths)

    for index in lindeces:
        if league_id == data.get("LeagueList.League%s.LeagueID" % index):
            return data.get("LeagueList.League%s.Season" % index)

    return None

def get_current_teamid(ret_userid = False):
    """Returns the current teamid or userid.

    If necessary, downloads a new team details xml to determine
    teamid and userid.

     - ret_userid -- if True, returns the userid instead of the teamid.
    """

    data = Database.databases.gets(None, False, ["Mattricks.current_teamid",
                                                 "Mattricks.current_userid"])
    teamid = data.get("Mattricks.current_teamid")
    userid = data.get("Mattricks.current_userid")

    if teamid is None or userid is None or len(teamid) == 0 or len(userid) == 0:
        data = XMLParse.get(["team details", -1], ["Team.TeamID", "User.UserID"])

        teamid = data.get("Team.TeamID")
        userid = data.get("User.UserID")

        if teamid is not None and userid is not None:
            Database.databases.sets(None, False, {"Mattricks.current_teamid": str(teamid),
                                                  "Mattricks.current_userid": str(userid)})
    else:
        teamid = int(teamid)
        userid = int(userid)

    if ret_userid:
        return userid
    return teamid

def is_supporter():
    """Returns true if the current user is a supporter"""
    data = Database.databases.gets(None, False, ["Mattricks.current_is_supporter"])
    supporter = data.get("Mattricks.current_is_supporter")

    if supporter is None or len(supporter) == 0:
        supporter = XMLParse.get(["team details", -1], ["User.HasSupporter"])
        if supporter is not None:
            Database.databases.sets(None, False, {"Mattricks.current_is_supporter": str(supporter)})
    else:
        supporter = string2bool(supporter)

    return supporter

    
