# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""The main frames and main loops of the graphical interface
of Mattricks."""

from Common import *
from Commonwx import *

import os
import os.path
import time
import sys

from Config import config
import Errorwx
import HRFData
import AboutPage
import Playerlist
import ComparisonPage
import Fetchdialog
import Fetch
import EconomyPage
import LeaguePage
import LineupPage
import MatchPrediction
import MatchViewerPage
import SummaryPage
import JuniorSpy
import TeamSelectorPage
import Configdialog
import DialogSet
import Database

from Images import Images

from wxPython.wx import *

notebook_pages = {}

class MainFrame(wxFrame):
    """The main windows of Mattricks."""

    ID_FETCH = 101
    ID_REFETCH = 102
    ID_EXIT  = 103
    ID_CONFIG = 104
    ID_OPEN = 105

    def __init__(self, parent, ID, title, filename):
        """Initialize the main window.

         - parent -- the parent of the frame.
         - ID -- the id of the frame.
         - title -- the title of the window.
         - filename -- a string containing a filename."""

        global notebook_pages

        wxFrame.__init__(self, parent, ID, title)

        if filename != None:
            config.set("Fetch", "save directory", os.path.dirname(filename))

        sizer = wxBoxSizer(wxVERTICAL)

        hsizer = wxBoxSizer(wxHORIZONTAL)

        buttons = [(MainFrame.ID_FETCH, _("Download data")),
                   (MainFrame.ID_REFETCH, _("Redownload data")),
                   (MainFrame.ID_CONFIG, _("Preferences")),
                   (MainFrame.ID_OPEN, _("Open data file")),
                   (MainFrame.ID_EXIT, _("Exit"))]
        for id, info in buttons:
            button = wxButton(self, id, info)
            hsizer.Add(button, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 0)

        sizer.AddSizer(hsizer, 0, wxEXPAND | wxBOTTOM, 0)

        notebook = wxNotebook(self, -1)
        nbsizer = wxNotebookSizer(notebook)

        the_pages = [
            ("about", "Mattricks", AboutPage.AboutPage(notebook, -1)),
            ("summary", _("Summary"), SummaryPage.SummaryPage(notebook, -1, filename)),
            ("player", _("Player lists"), Playerlist.PlayerPage(notebook, -1, filename)),
            ("comparison", _("Comparisons"), ComparisonPage.ComparisonPage(notebook, -1)),
            ("economy", _("Economy"), EconomyPage.EconomyPage(notebook, -1, filename)),
            ("table", _("League table"), LeaguePage.LeaguePage(notebook, -1)),
            ("lineup", _("Lineup"), LineupPage.LineupPage(notebook, -1, filename)),
            ("lineupselect", _("Lineup selection"),
             TeamSelectorPage.TeamSelectorPage(notebook, -1, filename)),
            ("prediction", _("Game prediction"), MatchPrediction.MatchPrediction(notebook, -1)),
            ("matchview", _("Match viewer"), MatchViewerPage.MatchViewerPage(notebook, -1))
            ]
        self.pages = {}
        notebook_pages = self.pages
        for name, descr, page in the_pages:
            notebook.AddPage(page, descr)
            self.pages[name] = page

        sizer.AddSizer(nbsizer, 1, wxEXPAND | wxALL, 0)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        EVT_BUTTON(self, MainFrame.ID_OPEN, self.OnOpen)
        EVT_BUTTON(self, MainFrame.ID_FETCH, self.OnFetch)
        EVT_BUTTON(self, MainFrame.ID_CONFIG, self.OnConfig)
        EVT_BUTTON(self, MainFrame.ID_REFETCH, self.OnRefetch)
        EVT_BUTTON(self, MainFrame.ID_EXIT,  self.OnExit)

        icons = wxIconBundle()
        for iconsize in [16, 32, 48, 64]:
            iconbmp = Images.get_bitmap_from_file("icon-%dx%d.png" % (iconsize, iconsize))
            icon = wxEmptyIcon()
            icon.CopyFromBitmap(iconbmp)
            icons.AddIcon(icon)
        self.SetIcons(icons)

    def AfterInit(self):
        """Initialization that needs to be done after __init__() but
        before everything else.

        This function also calls all pages in the notebook to after-
        initialize them."""
        if config.get("Fetch", "download at startup"):
            loginname = Database.databases.get(None, True, "Mattricks.loginname")
            password = Database.databases.get(None, True, "Mattricks.password")
            default_dir = config.get("Fetch", "save directory")
            default_jspy = config.get("Fetch", "junior spy")
            default_adownload = config.get("Fetch", "always download")
            default_override = config.get("Fetch", "override old hrf")
            default_server = config.get("Fetch", "ip number")
            self.__do_fetch(True,
                            default_dir, loginname, password,
                            default_jspy, default_adownload,
                            default_override, default_server)

        for name, page in self.pages.items():
            if getattr(page, "AfterInit", None) is not None:
                page.AfterInit()

        return True

    def OnOpen(self, event):
        """Called when the user wants to open a hrf on harddisk."""
        wildcard = "HRF Files (*.hrf)|*.hrf|" \
                   "All files (*.*)|*.*"

        default_dir = config.get("Fetch", "save directory")

        dlg = wxFileDialog(self, _("Choose file with hattrick data."), wildcard = wildcard,
                           style = wxOPEN, defaultDir = default_dir)
        if dlg.ShowModal() == wxID_OK:
            self.__change_hrf(dlg.GetPath())
        dlg.Destroy()

    def OnFetch(self, event):
        """Called when the user wants to download a hrf."""
        default_dir = config.get("Fetch", "save directory")
        default_jspy = config.get("Fetch", "junior spy")
        default_adownload = config.get("Fetch", "always download")
        default_override = config.get("Fetch", "override old hrf")
        default_server = config.get("Fetch", "ip number")
        loginname = Database.databases.get(None, True, "Mattricks.loginname")
        password = Database.databases.get(None, True, "Mattricks.password")
        self.__do_fetch(True,
                        default_dir, loginname, password,
                        default_jspy, default_adownload,
                        default_override, default_server)

    def OnRefetch(self, event):
        """Called when the user wants to download a hrf without a dialog."""
        default_dir = config.get("Fetch", "save directory")
        default_jspy = config.get("Fetch", "junior spy")
        default_adownload = config.get("Fetch", "always download")
        default_override = config.get("Fetch", "override old hrf")
        default_server = config.get("Fetch", "ip number")
        loginname = Database.databases.get(None, True, "Mattricks.loginname")
        password = Database.databases.get(None, True, "Mattricks.password")
        self.__do_fetch(False,
                        default_dir, loginname, password,
                        default_jspy, default_adownload,
                        default_override, default_server)

    def OnExit(self, event):
        """Called when the user wants to exit the program."""
        if config.get("General", "warning on exit") == True:
            closeDialog = wxMessageDialog(self, _("Exit Mattricks?"), _("Exit Mattricks?"), wxYES_NO)
            ans = closeDialog.ShowModal()
            if ans == wxID_NO:
                return

        self.Close(True)

    def GetTabHandle(self, tabname):
        """Get a page from the notebook by it's name."""
        return self.pages[tabname]

    def OnConfig(self, event):
        """Called when the user wants to configure Mattricks."""
        if not Configdialog.ConfigDialog.conf_dialog_running:
            dlg = Configdialog.ConfigDialog(self, -1)
            dlg.Show()

    def __change_hrf(self, hrffile):
        """Changes the currently used hrf. Also calls all pages"""
        hrfdir = os.path.dirname(hrffile)
        HRFData.htdirs.add_from_filename(hrffile)
        for name, page in self.pages.items():
            page.ChangeHRF(hrffile)

    def __do_fetch(self, use_dialog,
                   path, loginname, password, juniorspy,
                   always_download, overwrite_old, server):
        """Fetching with or without a login dialog."""

        exit = False
        hrffile = None
        if use_dialog:
            dlg = Fetchdialog.FetchDialog(self, _("Download data"),
                                          defaultDir = path,
                                          defaultUser = loginname,
                                          defaultPass = password,
                                          defaultJSpy = juniorspy,
                                          defaultADownload = always_download,
                                          defaultOverride = overwrite_old)

            while not exit:
                res = dlg.ShowModal()
                if res == wxID_OK:
                    args = dlg.GetResult() + (server,)
                    juniorspy = args[3]
                    (lname, pword) = args[1:3]
                    Database.databases.sets(None, True, {"Mattricks.loginname": lname,
                                                         "Mattricks.password": pword})
                    (exit, hrffile) = self.__do_fetch_internal(*args)
                else:
                    exit = True
            dlg.Destroy()
        else:
            (exit, hrffile) = self.__do_fetch_internal(path, loginname, password,
                                                       juniorspy, always_download,
                                                       overwrite_old, server)

        if hrffile != None:
            if juniorspy == True:
                psdlg=JuniorSpy.JuniorSpySelect(self,HRFData.hrfstorage[hrffile])
                psdlg.ShowModal()
            self.__change_hrf(hrffile)

    def __do_fetch_internal(self, path, loginname, password, juniorspy,
                            always_download, overwrite_old, server):
        """Does the actual fetching by calling Fetch.conn."""

        hrffile = None
        exit = False
        err_message = None
        pddlg = None
        if loginname is None or len(loginname) == 0 or \
               password is None or len(password) == 0:
            err_message = _("You have to supply both username and password")
        else:
            pddlg = wxProgressDialog(_("Fetching hrf file ..."),
                                     _("Logging in ..."),
                                     10,
                                     self,
                                     wxPD_APP_MODAL)

            self.default_user = loginname
            self.default_pass = password
            try:
                result = Fetch.conn.fetch_hrf(loginname, password, path, pddlg, None,
                                              always_download, overwrite_old, server)
                hrffile = result
                exit = True
            except Fetch.FetchError, e:
                err_message = str(e)
                if e.type != Fetch.FETCH_ERROR_LOGIN_ERROR:
                    exit = True

        if pddlg != None:
            pddlg.Destroy()

        if err_message != None:
            mdlg = wxMessageDialog(self, err_message,
                                  _("Error"), wxOK | wxICON_ERROR)
            mdlg.ShowModal()
            mdlg.Destroy()

        return (exit, hrffile)

class Main(wxApp):
    """The main application of the program."""

    ID_FRAME = 201
    def __init__(self, filename = None):
        """First initialization."""

        self.initial_file = filename
        wxApp.__init__(self)

    def OnInit(self):
        """Create windows and change to graphical dialogs."""
        frame = MainFrame(NULL, Main.ID_FRAME, "Mattricks", self.initial_file)
        self.SetTopWindow(frame)
        frame.Show(True)

        Errorwx.error.init_errorwx(frame)
        DialogSet.progress_dialog_maker_func = \
                  (lambda p=frame: wxProgressDialog(_("Fetching data ..."),
                                                    _("Logging in ..."),
                                                    10,
                                                    p,
                                                    wxPD_APP_MODAL))
        DialogSet.destroy_dialog_maker_func = (lambda dlg: dlg.Destroy())
        DialogSet.password_getting_dialog = (lambda p=frame: Fetchdialog.graphical_password_dialog(p))
        DialogSet.error_message_dialog = (lambda msg, p=frame: Fetchdialog.graphical_error_msg_dialog(p, msg))

        del self.initial_file

        frame.AfterInit()
        return true

def main(*hrfs):
    """The entry point for the graphical version of Mattricks.

     - *hrfs -- an amount of strings of filenames pointing to hrfs."""
    filename = None
    if len(hrfs) == 0:
        hrfs = config.get("General", "hrf directories")
    for hrf in hrfs:
        hrf = os.path.realpath(os.path.expanduser(hrf))
        hrfdir = os.path.dirname(hrf)
        if os.path.isdir(hrf):
            HRFData.htdirs.add_dirs(hrf)
        else:
            HRFData.htdirs.add_from_filename(hrf)
            filename = hrf

    if filename == None:
        filename = HRFData.htdirs.get_from_time(time.time())

    app = Main(filename)
    app.MainLoop()
